#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include "jtypes.h"

// Definitions :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

#ifndef CODER
#define CODER               Beppu
#endif

#define LONGEST_FILENAME    255
#define HSI_HEADER_LEN      32
#define PALETTE_SET         1

// Classes && Structures :::::::::::::::::::::::::::::::::::::::::::::::::::

class SPRFile {

public:

    // [con,de]struction ...................................................

    SPRFile();
    virtual ~SPRFile();

    // querying ............................................................

    virtual u32 spriteQty();
    virtual u32 sizeOfHSI (
        u32 spriteNDX
        ) const = 0;
    virtual u32 spriteWd (
        u32 spriteNDX
        ) const = 0;
    virtual u32 spriteHi (
        u32 spriteNDX
        ) const = 0;

    // manipulation ........................................................

    virtual void makeHSI (
        u8 *dst,
        u32 spriteNDX
        ) const = 0;
    virtual void blt8bpp (
        u8 *dst,
        u32 pitch,
        u32 spriteNDX
        ) const = 0;
    void definePalette (
        const u8 *pal,
        u32 len
        );

protected:

    u32         spriteQTY;
    u32         flags;
    u32         palLEN;
    u8          palBUF[768];

};


// Methods :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


/*
    [function]  first step of object creation
`.....................................................................*/
SPRFile::SPRFile()
{
    spriteQTY = 0;
    flags = 0;
    palLEN = 0;
}


/*
    [function]  virtual destructor
`.....................................................................*/
SPRFile::~SPRFile()
{
    // exists to avoid future deallocation errors.
}


/*
    [returns]   quantity of sprites
`.....................................................................*/
u32 SPRFile::spriteQty()
{
    return spriteQTY;
}


/*
    [function]  Sets palette for set of sprites found in an
                SPRFile derivative
    [throws]
`.....................................................................*/
void SPRFile::definePalette (
    const u8 *pal,
    u32 len
    )
{
    if (len == 0) return;
    if (len > 256) return;
    memcpy(palBUF,pal,(len * 3));
    flags |= PALETTE_SET;
    palLEN = len;
}
